//created by Alex Hartley
//This script creates nulls at the locations of the selected items, 
//and it provides options for how those nulls should be related to
//the original items.

@warnings

//global variables
selection,
prefix,
suffix,
findString,
rowWidth,
rowHeight,
section,
padding,
spacing,
replaceString;


generic
{
	//get scene info.
    scene = Scene();
    time = scene.currenttime;
	
    // get rid of ikfkMaster if it exists
	servers = scene.server( SERVER_MASTER_H );
	i = servers.size();
	while( i )
	{
		if (servers[i] == "AH_ikfkMaster")
			RemoveServer(SERVER_MASTER_H, i);
		i--;
	}

	// errors
	if (!scene.generalopts[3])
		error( "'Parent in Place' must be enabled.");

	//load settings

	prefix = recall("AH_makeNulls_prefix");
	if(!prefix)
		prefix="";
	suffix = recall("AH_makeNulls_suffix");
	if(!suffix)
		suffix="";

	findString = recall("AH_makeNulls_findString");
	if(!findString)
		findString="";

	replaceString = recall("AH_makeNulls_replaceString");
	if(!replaceString)
		replaceString="";

	parenting = recall("AH_makeNulls_parenting");
	if(!parenting)
		parenting=false;
	
	positionItem = recall("AH_makeNulls_positionItem");
	if(!positionItem)
		positionItem=false;

	rotationItem = recall("AH_makeNulls_rotationItem");		
	if(!rotationItem)
		rotationItem=false;

	scaleItem = recall("AH_makeNulls_scaleItem");
	if(!scaleItem)
		scaleItem=false;
	localWorld = recall("AH_makeNulls_localWorld");
	if(!localWorld)
		localWorld=2;
	envelope = recall("AH_makeNulls_envelope");		
	if(!envelope)
		envelope=false;

	relationship = recall("AH_makeNulls_relationship");		
	if(!relationship)
		relationship=1;

	direction = recall("AH_makeNulls_direction");		
	if(!direction)
		direction=1;

	replaceConstraints = recall("AH_makeNulls_replaceConstraints");				
	if(!replaceConstraints)
		replaceConstraints=false;

	ignoreConstrained = recall("AH_makeNulls_ignoreConstrained");				
	if(!ignoreConstrained)
		ignoreConstrained=false;

	itemColor = recall("AH_makeNulls_itemColor");	
	if(!itemColor)
		itemColor=8;
				
	visibility = recall("AH_makeNulls_visibility");				
	if(!visibility)
		visibility=1;

	lock = recall("AH_makeNulls_lock");				
	if(!lock)
		lock=1;

    //requester
    rowWidth = 250;
    rowHeight = 20;
    padding = 8;
    spacing = 8;
    reqbegin("makeNulls");
    x = padding;
    y = padding;
    s=1;

	section[s]="Naming_"+string(x)+"_"+string(y);
	s++;
    y = y + rowHeight + 2*spacing;


    cPrefix = ctlstring("Null Prefix", prefix);
    
    ctlposition(cPrefix, x, y, rowWidth, rowHeight);
    y = y + rowHeight + spacing;
    
	cSuffix = ctlstring("Null Suffix", suffix);
    ctlposition(cSuffix, x, y, rowWidth, rowHeight);
    y = y + rowHeight + spacing;


	cFind = ctlstring("Find String", findString);
    ctlposition(cFind, x, y, rowWidth, rowHeight);
    y = y + rowHeight + spacing;


	cReplace = ctlstring("Replace String", replaceString);
    ctlposition(cReplace, x, y, rowWidth, rowHeight);
    y = y + rowHeight + spacing;


	ctlactive(cFind,"showReplace",cReplace);
	
	section[s]="Relationship_"+string(x)+"_"+string(y);
	s++;
    y = y + rowHeight + 2*spacing;

	cRelationship= ctlpopup("",relationship,@"None","New nulls are parents of selected items","New nulls are children of selected items"@);
	ctlposition(cRelationship, x - 6, y, rowWidth + 6, rowHeight);
    y = y + rowHeight + spacing;

	section[s]="Null Hierarchy_"+string(x)+"_"+string(y);
	s++;
    y = y + rowHeight + 2*spacing;

    cParenting = ctlcheckbox("replicate selection hierarchy", parenting);
    ctlposition(cParenting, x, y, rowWidth, rowHeight);
    y = y + rowHeight + spacing;

	ctlactive(cRelationship,"showExtra",cParenting);

	section[s]="Same As Item Constraints_"+string(x)+"_"+string(y);
	s++;
    y = y + rowHeight + 2*spacing;
    
	cDirection = ctlpopup("",direction,@"New nulls constrain selected items","Selected items constrain new nulls"@);
	ctlposition(cDirection, x - 6, y, rowWidth + 6, rowHeight);
    y = y + rowHeight + spacing;
	
	ctlactive(cRelationship,"showExtra",cDirection);
	//position controls
	cPositionItem = ctlcheckbox("Position", positionItem);
    ctlposition(cPositionItem, x, y, rowWidth/3, rowHeight);


	ctlactive(cRelationship,"showExtra",cPositionItem);
	//rotation controls
	cRotationItem = ctlcheckbox("Rotation", rotationItem);
    ctlposition(cRotationItem, x+rowWidth/3, y, rowWidth/3, rowHeight);
	
	
	ctlactive(cRelationship,"showExtra",cRotationItem);
	//scale controls	
	cScaleItem = ctlcheckbox("Scale", scaleItem);
	ctlposition(cScaleItem, x+2*rowWidth/3, y, rowWidth/3, rowHeight);
    y = y + rowHeight + spacing;

	
	ctlactive(cRelationship,"showExtra",cScaleItem);
	
	//localWorld toggle
	cLocalWorld = ctlchoice("",localWorld,@"Local","World"@);
	ctlposition(cLocalWorld, x - 6, y, rowWidth + 6, rowHeight);
    y = y + rowHeight + spacing;
	
	
	ctlactive(cRelationship,"showExtra",cLocalWorld);

	
	//envelope toggle
	cEnvelope = ctlcheckbox("Add envelopes", envelope);
    ctlposition(cEnvelope, x, y, rowWidth, rowHeight);
    y = y + rowHeight + spacing;
	
	
	ctlactive(cRelationship,"showExtra",cEnvelope);

	
	//replace existing constraints
	cReplaceConstraints = ctlcheckbox("Replace existing constraints", replaceConstraints);
    ctlposition(cReplaceConstraints, x, y, rowWidth, rowHeight);
    y = y + rowHeight + spacing;
	
	ctlactive(cRelationship,"showExtra",cEnvelope);

	//create constrained?
	cIgnoreConstrained = ctlcheckbox("Skip position constrained items", ignoreConstrained);
    ctlposition(cIgnoreConstrained, x, y, rowWidth, rowHeight);
    y = y + rowHeight + spacing;
	
	ctlactive(cRelationship,"showExtra",cEnvelope);

	section[s]="Visibility_"+string(x)+"_"+string(y);
	s++;
    y = y + rowHeight + 2*spacing;

	//visibility 
	cColor = ctlpopup("Null Color",itemColor,@"Black","Dark Blue","Dark Green","Dark Cyan","Dark Red","Purple","Brown","Grey","Blue","Green","Cyan","Red","Magenta","Orange","White"@);
	ctlposition(cColor, x, y, rowWidth, rowHeight);
    y = y + rowHeight + spacing;

	//visibility 
	cVisibility = ctlchoice("",visibility,@"Hide none","Hide new nulls","Hide selection"@);
	ctlposition(cVisibility, x - 6, y, rowWidth + 6, rowHeight);
    y = y + rowHeight + spacing;

	//locking 
	cLock = ctlchoice("",lock,@"Lock none","Lock new nulls","Lock selection"@);
	ctlposition(cLock, x - 6, y, rowWidth + 6, rowHeight);
    y = y + rowHeight + spacing;


	reqredraw("draw");
	reqsize(rowWidth+padding*2,y+padding + rowHeight);
    return if !reqpost();

	if (reqpost())
	{		

		// get user settings		
		prefix = getvalue(cPrefix);
		suffix = getvalue(cSuffix);
		findString = getvalue(cFind);
		replaceString = getvalue(cReplace);
		parenting = getvalue(cParenting);
		positionItem = getvalue(cPositionItem);
		rotationItem = getvalue(cRotationItem);
		scaleItem = getvalue(cScaleItem);
		localWorld = getvalue(cLocalWorld);
		envelope = getvalue(cEnvelope);
		relationship = getvalue(cRelationship);
		direction = getvalue(cDirection);
		replaceConstraints = getvalue(cReplaceConstraints);
		ignoreConstrained = getvalue(cIgnoreConstrained);
		itemColor = getvalue(cColor);
		visibility = getvalue(cVisibility);
		lock = getvalue(cLock);

		//store settings
		store("AH_makeNulls_prefix", prefix);
		store("AH_makeNulls_suffix", suffix);
		store("AH_makeNulls_findString", findString);
		store("AH_makeNulls_replaceString", replaceString);
		store("AH_makeNulls_parenting", parenting);
		store("AH_makeNulls_positionItem", positionItem);
		store("AH_makeNulls_rotationItem", rotationItem);		
		store("AH_makeNulls_scaleItem", scaleItem);
		store("AH_makeNulls_localWorld", localWorld);
		store("AH_makeNulls_envelope", envelope);		
		store("AH_makeNulls_relationship", relationship);		
		store("AH_makeNulls_direction", direction);		
		store("AH_makeNulls_replaceConstraints", replaceConstraints);				
		store("AH_makeNulls_ignoreConstrained", ignoreConstrained);				
		store("AH_makeNulls_itemColor", itemColor);				
		store("AH_makeNulls_visibility", visibility);				
		store("AH_makeNulls_lock", lock);				

		//get the selection
		selection = scene.getSelect();
		
		//reduce selection if constrained
		if(ignoreConstrained==true)
		{
			j=1;
			for(i=1; i<=selection.size(); i++)
			{
				if(selection[i].controller[1] == 1)
				{
					unconstrainedSelection[j] = selection[i];
					j++;
				}
			}		
			selection = nil;
			selection = unconstrainedSelection;
			if(unconstrainedSelection == nil)
				error("all of the selected nulls have position constraints, and you have chosen to skip constrained items. No nulls created from these settings");
			unconstrainedSelection = nil;
		}
		
	
		if (prefix == "" && suffix == "" && findString == "")
		error("must have a prefix, suffix or find and replace defined ");
	
		//Check that the null names are unique
		for(i=1; i<=selection.size(); i++)
		{
			nullName = getNullName(selection[i]);
			SelectAllObjects();
			allItems = scene.getSelect();
				for(j=1; j<=allItems.size(); j++)
				{
					if(nullName == allItems[j].name )
						error(nullName + " already exists");
				}
			//clear variables             
			nullName= nil;
		}
		//Make the nulls
		for(i=1; i<=selection.size(); i++)
		{
			wPos[i] = selection[i].getWorldPosition(time);
			wRot[i] = selection[i].getWorldRotation(time);
			scale[i] = selection[i].getScaling(time);
			nullName = getNullName(selection[i]);
			AddNull(nullName);

			//set the null visibility settings
			ItemColor(itemColor);
			if(lock==2)
				ItemLock(1);
			if(visibility==2)
				ItemVisibility(1);

			//set the selection visibility settings	
			if(lock==3 || visibility==3)
			{
				selection[i].select();
				if(lock==3)
					ItemLock(1);
				if(visibility==3)
					ItemVisibility(1);
			}			
		}
		//position the nulls
		for(i=1;i<=selection.size();i++)
		{
			nullName = nil;
			nullName = getNullName(selection[i]);
			SelectItem(nullName);
			Position(wPos[i]);
			Rotation(wRot[i]);
			Scale(scale[i]);
			UpdateMotion();
			CreateKey(0);
		}
	
		// set up parenting on nulls if clone parenting is enabled.
		if(parenting)
		{
			// check that the parent exists
			for(i=1; i<=selection.size(); i++) 
			{
				if (selection[i].parent)
				{
					parent = selection[i].parent;
					parentName = getNullName(selection[i].parent);
					nullName = getNullName(selection[i]);	
							
					//check that the parent is among the original selection, and if there is a match do reparenting.
					for(j=1; j<=selection.size(); j++)
					{
						if(parent.name == selection[j].name)
						{
							SelectItem(nullName);
							ParentItem(parentName);
						}
					}
				
					//clear variables             
					parentName = nil;
					nullName= nil;			
					parent = nil;					
				}
			}	    
		}
		//setup parenting based on relationship setting
		{
			for(i=1; i<=selection.size(); i++) 
			{
				parent = selection[i].parent;
				nullName = getNullName(selection[i]);	
				if(relationship==2)
				{
					if(selection[i].isBone())
					{
						error( "\"New nulls are parents of selected items\" relationship is not allowed if your selection is bones");
					}
					else
					{
						SelectItem(selection[i]);
						ParentItem(nullName);
						if(parent != nil)
						{
							SelectItem(nullName);
							ParentItem(parent.name);
						}
					}
				
				}
				if(relationship==3)
				{
						SelectItem(nullName);
						ParentItem(selection[i].name);
				}
				//clear variables
				nullName= nil;
				parent=nil;	
			}
		}
	
		//set up same as item if selected
	
		for(i=1; i<=selection.size(); i++)
		{
			//check controllers
			pConstraints = nil;
			rConstraints = nil;
			sConstraints = nil;
			for(j=1; j<=9; j++)
			{
				if( j <= 3 && selection[i].controller[j] != 1 )
					pConstraints = true;
				else if( j <= 6 && selection[i].controller[j] != 1 )
					rConstraints = true;
				else if( selection[i].controller[j] != 1 )
					sConstraints = true;
			}
			if(direction == 1)
			{
				nullName = getNullName(selection[i]);	
				selection[i].select();
				if(positionItem && !pConstraints || replaceConstraints )
				{	
					PositionItem(nullName);
					if(localWorld==2)
						PositionItemWorld(1);
					else
						PositionItemWorld(0);
					XController(7);
					YController(7);
					ZController(7);
					if(envelope)
						AddEnvelope("Ctrl.PosItemBlend");			
				}
		
				if(rotationItem && !rConstraints || replaceConstraints )
				{
					RotationItem(nullName);
					if(localWorld==2)
						RotationItemWorld(1);
					else
						RotationItemWorld(0);
					HController(7);
					PController(7);
					BController(7);
					if(envelope)				
						AddEnvelope("Ctrl.RotItemBlend");
				}

				if(scaleItem && !sConstraints || replaceConstraints )
				{
					ScaleItem(nullName);
					SXController(7);
					SYController(7);
					SZController(7);
					if(envelope)				
						AddEnvelope("Ctrl.ScaleItemBlend");	
				}		
				// clear variables
				nullName= nil;			
			}

			if(direction == 2)
			{
				nullName = getNullName(selection[i]);	
				SelectItem(nullName);
				if(positionItem)
				{	
					PositionItem(selection[i].id);
					if(localWorld==2)
						PositionItemWorld(1);
					else
						PositionItemWorld(0);
					XController(7);
					YController(7);
					ZController(7);
					if(envelope)
						AddEnvelope("Ctrl.PosItemBlend");			
				}
		
				if(rotationItem)
				{
					RotationItem(selection[i].id);
					if(localWorld==2)
						RotationItemWorld(1);
					else
						RotationItemWorld(0);

					HController(7);
					PController(7);
					BController(7);
					if(envelope)				
						AddEnvelope("Ctrl.RotItemBlend");
				}

				if(scaleItem)
				{
					ScaleItem(selection[i].id);
					SXController(7);
					SYController(7);
					SZController(7);
					if(envelope)				
						AddEnvelope("Ctrl.ScaleItemBlend");	
				}		
				// clear variables
				nullName= nil;			
			}
		}
		//Select the New Nulls for the user
		{
		
			for(i=1; i<=selection.size(); i++)
			{
				nullName = getNullName(selection[i]);	
				if (i == 1)
				{
					SelectItem(nullName);
				}
				else
				{
					AddToSelection(nullName);
				}
					
				// clear variables
				nullName= nil;			
			}	
		}
	}
}
draw
{
	for(i=1;i<=section.size();i++)
	{
		parts = parse("_", section[i]);
		header = parts[1];
		x = integer(parts[2]);
		y = integer(parts[3])+ spacing;
		color= <200,200,200>;
		color2= <100,100,100>;
		drawtext(header, color,x,y);
		drawline(color2,0,y + rowHeight, rowWidth+padding*2,y + rowHeight);
	}

}
showExtra : value
{
	return(value == 1);
}
showReplace : value
{
	return(value != "");
}
getNullName : item
{
	nullName = nil;
	count=0;
	if(findString != "")
	{
		nullName = item.name;
		for(i=1; i<=nullName.size(); i++)
		{
			if ( strsub(nullName, i, findString.size()) == findString )
			{
				nullName = strleft( nullName , i - 1 ) + replaceString + strright( nullName , nullName.size() - findString.size() - i + 1 );
				i= i - 1 + replaceString.size();
				count++;
			}
		}	
		if(count == 0)
		error("findString is not present in " + item.name);
		nullName = prefix + nullName + suffix;		
	}
	else 
		nullName = prefix + item.name + suffix;
		return(nullName);
		
}